/*!
* unplugin-vue-component-ref v1.0.3
* (c) 2022 zrrz
* @license MIT
*/
import { createUnplugin } from 'unplugin';
import { createFilter } from '@rollup/pluginutils';
import { parse, MagicString, compileTemplate } from '@vue/compiler-sfc';
import { findProp } from '@vue/compiler-core';

const onVnodeBeforeMountRef = ' :onVnodeBeforeMount="onVnodeBeforeMountRef_" ';

const transform = (source, filename) => {
    const { descriptor } = parse(source);
    if (descriptor.template) {
        const { loc: templateLocStart } = descriptor.template;
        const magicString = new MagicString(source);
        const onRefNode = (node) => {
            if (node.type === 1 /** NodeTypes.ELEMENT */) {
                const ref = findProp(node, 'ref');
                if (ref && ref.type === 6 /**  NodeTypes.ATTRIBUTE */) {
                    const { loc } = ref;
                    magicString.appendLeft(loc.end.offset + templateLocStart.start.offset, onVnodeBeforeMountRef);
                }
            }
        };
        compileTemplate({
            filename,
            source: descriptor.template.content,
            id: filename,
            compilerOptions: {
                nodeTransforms: [onRefNode]
            }
        });
        return magicString.toString();
    }
};

const onVnodeBeforeMountRef_ = (VNode) => {
    const { component } = VNode;
    if (component) {
        component.exposeProxy = new Proxy({}, {
            get(_, key) {
                if (component.exposed && key in component.exposed)
                    return component.exposed[key];
                //@ts-ignore
                return component.proxy[key];
            }
        });
    }
};
var client = (app) => {
    app.config.globalProperties.onVnodeBeforeMountRef_ = onVnodeBeforeMountRef_;
};

function resolveOption(options) {
    return {
        include: options.include || [/\.vue$/],
        exclude: options.exclude || undefined
    };
}
const unplugin = createUnplugin((userOptions) => {
    const options = resolveOption(userOptions);
    const filter = createFilter(options.include, options.exclude);
    return {
        name: 'unplugin-vue-component-ref',
        enforce: "pre",
        transformInclude(id) {
            return filter(id);
        },
        transform(code, id) {
            if (filter(id))
                transform(code, id);
            return null;
        }
    };
});
const vitePlugin = unplugin.vite;
const rollupPlugin = unplugin.rollup;
const webpackPlugin = unplugin.webpack;
const esbuildPlugin = unplugin.esbuild;

export { esbuildPlugin, rollupPlugin, unplugin, client as unpluginVueRef, vitePlugin, webpackPlugin };
